/*
Storybook: Open Source software for novelists and authors.
Copyright (C) 2008 - 2012 Martin Mustun

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package storybook.ui.panel.navigation;

import java.awt.event.ActionEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.beans.PropertyChangeEvent;
import java.util.Date;
import java.util.List;
import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JPanel;
import javax.swing.Timer;
import org.hibernate.Session;
import org.miginfocom.swing.MigLayout;
import resources.icons.ICONS;
import resources.icons.IconButton;
import storybook.action.ScrollToStrandDateAction;
import i18n.I18N;
import storybook.model.EntityUtil;
import storybook.model.Model;
import storybook.model.book.Book;
import storybook.model.hbn.dao.SceneDAOImpl;
import storybook.model.hbn.entity.Strand;
import storybook.toolkit.swing.SwingUtil;
import storybook.toolkit.swing.js.JSLabel;
import storybook.toolkit.swing.js.JSPanelViewsRadioButton;
import storybook.ui.MIG;
import storybook.ui.MainFrame;
import storybook.ui.SbView;
import storybook.ui.SbView.VIEWNAME;
import storybook.ui.panel.AbstractPanel;
import storybook.ui.panel.book.BookPanel;
import storybook.ui.panel.chrono.ChronoPanel;

/**
 * @author martin
 *
 */
@SuppressWarnings("serial")
public class FindDatePanel extends AbstractPanel implements ItemListener {

	private JComboBox strandCombo;
	private JComboBox dateCombo;
	private JSPanelViewsRadioButton viewsRbPanel;
	private JSLabel lbWarning;

	public FindDatePanel(MainFrame mainFrame) {
		super(mainFrame);
		initAll();
	}

	@Override
	public void modelPropertyChange(PropertyChangeEvent evt) {
		// empty
	}

	@Override
	public void init() {
		// empty
	}

	@Override
	public void initUi() {
		setLayout(new MigLayout(MIG.get(MIG.FILLX, MIG.WRAP + " 2"), "[]10[grow]", "[][]10[]"));
		add(new JSLabel(I18N.getColonMsg("strand")));
		strandCombo = new JComboBox();
		EntityUtil.fillCB(mainFrame, strandCombo, Book.TYPE.STRAND, new Strand(), false, false);
		strandCombo.addItemListener(this);
		add(strandCombo, MIG.GROWX);
		add(new JSLabel(I18N.getColonMsg("date")));
		dateCombo = new JComboBox();
		refreshDateCombo();
		add(dateCombo, MIG.get(MIG.GROWX, "split 3"));
		IconButton btPrev = new IconButton("btPrev", ICONS.K.NAV_PREV, "previous", getPreviousAction());
		btPrev.set20x20();
		add(btPrev);
		IconButton btNext = new IconButton("btNext", ICONS.K.NAV_NEXT, "next", getNextAction());
		btNext.set20x20();
		add(btNext);
		add(new JSLabel(I18N.getColonMsg("navigation.show.in")), MIG.TOP);
		viewsRbPanel = new JSPanelViewsRadioButton(mainFrame, false);
		add(viewsRbPanel);
		lbWarning = new JSLabel(" ");
		add(lbWarning, MIG.get(MIG.SPAN, MIG.SPLIT2, MIG.LEFT, MIG.GROWX));
		JButton btFind = new IconButton("btFind", ICONS.K.SEARCH, getFindAction());
		btFind.setText(I18N.getMsg("find"));
		SwingUtil.addEnterAction(btFind, getFindAction());
		add(btFind, MIG.RIGHT);
	}

	@SuppressWarnings("unchecked")
	private void refreshDateCombo() {
		Strand strand = (Strand) strandCombo.getSelectedItem();
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		SceneDAOImpl dao = new SceneDAOImpl(session);
		List<Date> dates = dao.findDistinctDatesByStrand(strand);
		model.commit();
		dateCombo.removeAllItems();
		for (Date date : dates) {
			dateCombo.addItem(date);
		}
	}

	private AbstractAction getPreviousAction() {
		return new AbstractAction() {
			@Override
			public void actionPerformed(ActionEvent evt) {
				int index = dateCombo.getSelectedIndex();
				--index;
				if (index == -1) {
					index = 0;
				}
				dateCombo.setSelectedIndex(index);
				scrollToStrandDate();
			}
		};
	}

	private AbstractAction getNextAction() {
		return new AbstractAction() {
			@Override
			public void actionPerformed(ActionEvent evt) {
				int index = dateCombo.getSelectedIndex();
				++index;
				if (index == dateCombo.getItemCount()) {
					index = dateCombo.getItemCount() - 1;
				}
				dateCombo.setSelectedIndex(index);
				scrollToStrandDate();
			}
		};
	}

	private AbstractAction getFindAction() {
		return new AbstractAction() {
			@Override
			public void actionPerformed(ActionEvent evt) {
				scrollToStrandDate();
			}
		};
	}

	private void scrollToStrandDate() {
		Strand strand = (Strand) strandCombo.getSelectedItem();
		Date date = (Date) dateCombo.getSelectedItem();
		SbView view = null;
		boolean chrono = viewsRbPanel.isChronoSelected();
		boolean isBook = viewsRbPanel.isBookSelected();
		if (chrono) {
			view = mainFrame.getView(VIEWNAME.CHRONO);
			mainFrame.showView(VIEWNAME.CHRONO);
		} else if (isBook) {
			view = mainFrame.getView(VIEWNAME.BOOK);
			mainFrame.showView(VIEWNAME.BOOK);
		}
		AbstractPanel container = null;
		JPanel panel = null;
		if (chrono) {
			container = (ChronoPanel) view.getComponent();
			panel = ((ChronoPanel) container).getPanel();
		} else if (isBook) {
			container = (BookPanel) view.getComponent();
			panel = ((BookPanel) container).getPanel();
		}
		int delay = 20;
		if (view != null && !view.isLoaded()) {
			delay += 180;
		}
		if (isBook) {
			delay += 100;
		}
		ScrollToStrandDateAction action = new ScrollToStrandDateAction(
				container, panel, strand, date, lbWarning);
		Timer timer = new Timer(delay, action);
		timer.setRepeats(false);
		timer.start();
	}

	@Override
	public void itemStateChanged(ItemEvent e) {
		if (e.getStateChange() == ItemEvent.SELECTED) {
			refreshDateCombo();
		}
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		// empty
	}

}
